#ifndef __DEBUG_H__
#define __DEBUG_H__

#include <stdbool.h>

/** @file debug.h
 *  @brief Define a <b>simple</b> debugging interface to use in C programs
 *  @details One method of debugging your C program is to add <tt>printf()</tt>
 *  statements to your code. This file provides a way of including debug
 *  output, and being able to turn it on/off either at compile time or
 *  at runtime, without making <b>any</b> changes to your code.
 *  Two levels of debugging are provided. If
 *  the value <tt>DEBUG_ENABLED</tt> is defined, your debug calls are compiled
 *  into your code. Otherwise, they are omitted. There is an additional
 *  runtime check as to whether to actually print the debugging output. This
 *  is controlled by the "flag" <tt>debug_active</tt>.
 *  <p>
 *  To use it with <tt>gcc</tt>, simply write a <tt>printf()</tt> call, but
 *  replace the <tt>printf()</tt> call by <tt>DEBUG()</tt>.
 *  <p>
 *  To get more information use the macro <tt>dDEBUG</tt>. It is identical
 *  to <tt>DEBUG</tt>, but adds the file and line number to the output.
 *  <p>
 *  To easily print the value of a <b>single</b> variable, use
 *  <pre><tt>
    vDEBUG("format", var); // "format" is the specifier (e.g. "%d" or "%s", etc)
 *  </tt></pre>
 *  <p>
 *  Based on code and ideas found 
 *  <a href="http://stackoverflow.com/questions/1644868/c-define-macro-for-debug-printing">
 *  here</a> and
 *  <a href="http://stackoverflow.com/questions/679979/how-to-make-a-variadic-macro-variable-number-of-arguments">
 *  here</a>. 
 **/

/** Provide a <tt>printf()</tt> like function with a variable number of
 *  parameters. This function takes advantage of the facilities for variable
 *  number of parameters contained in <tt>stdarg.h</tt>.
 *  @param fmt - <b>required</b> format string identical to first parameter in
 *  <tt>printf()</tt>.
 *  @param ... - <b>optional</b> parameters. The number and type must correspond
 *  with the argument specifiers in the <tt>fmt</tt>.
 */

void dbg_printf(const char* fmt, ...);

/** flag for controlling runtime output of debugging */
extern bool debug_on;

#ifndef DEBUG_ENABLED //replace all debugging code with empty statement

#define DEBUG(fmt, ...)  ((void)0)
#define vDEBUG(fmt, var) ((void)0)
#define HERE             ((void)0)

#else // compile debugging into code, although it only conditionally executes

// The '##__VA_ARGS__' is a gcc extension for variable number of parameters

#define DEBUG(fmt, ...) dbg_printf(fmt, ##__VA_ARGS__) 

// debug a single variable value e.g. vDEBUG("%d\n", intVar)

#define vDEBUG(fmt, var) \
   dbg_printf("[%s:%d] %s: " fmt, __FILE__, __LINE__, #var, var)

// simple trace of where the program is
#define HERE \
   dbg_printf("[%s:%s():%d] HERE", __FILE__, __func__, __LINE__)

#endif

#endif
